program readsaf137

!     Description:
!
!     Read 137-level diverse profile dataset from ECMWF
!     Owner:
!     Frederic Chevallier
!
!     History:
!     Version      Date        Comment
!     1            22/11/2006  Frederic Chevallier, LSCE
!     1.1          02/09/2014  Update to 137 levels, Reima Eresmaa, ECMWF 
!
!     Code description:
!       Language:              Fortran 90.
!       Software Standards:    "European Standards for Writing and Documenting
!                              Exchangeable Fortran 90 code".

implicit none

integer, parameter      :: nlev = 137 ! number of vertical levels

real, dimension(nlev)   :: temp, hum, ozo, cc, clw, ciw
real, dimension(nlev)   :: rain, snow, w, pap
real, dimension(nlev+1) :: paph
real                    :: lnpsurf, psurf, z0, tsurf, t2m, td2m, hum2m, u10, v10
real                    :: stratrsrf, convrsrf, snowsurf, elev
real                    :: lsm, lat, long
real                    :: alb, sr, tvl, tvh, cvl, cvh, seaice
real                    :: asn, rsn, tsn, sd, stl1, stl2, stl3, stl4
real                    :: swvl1, swvl2, swvl3, swvl4, istl1, istl2, istl3, istl4
integer                 :: year, month, day, step, gpoint, ind, i
character(len=10)       :: cvar
!

write(*,*) "Enter the identification of the sampled variable:"
write(*,*) "- t (for temperature)"
write(*,*) "- q (for humidity)"
write(*,*) "- oz (for ozone)"
write(*,*) "- ccol (for cloud condensate)"
write(*,*) "- rcol (for precipitation)"
read(*,*) cvar
write(*,*) cvar

if (trim(cvar) .ne. 't'    .and. trim(cvar) .ne. 'q'   .and. trim(cvar) .ne. 'oz' .and. &
    trim(cvar) .ne. 'ccol' .and. trim(cvar) .ne. 'rcol') then
  write(*,*) 'Wrong variable, start again'
  stop
endif

open(1,file='nwp_saf_'//trim(cvar)//'_sampled.atm',form='formatted')
open(2,file='nwp_saf_'//trim(cvar)//'_sampled.sfc',form='formatted')

i = 0
do
  read(1,'(1246(e13.6,x),i5,3i3,i7,i5)',end=1) &
     temp(:),     &! 1) Temperature [K]                          (1-137)
     hum(:),      &! 2) Humidity [kg/kg]                         (138-274)
     ozo(:),      &! 3) Ozone [kg/kg]                            (275-411)
     cc(:),       &! 4) Cloud Cover [0-1]                        (412-548)
     clw(:),      &! 5) C Liquid W [kg/kg]                       (549-685)
     ciw(:),      &! 6) C Ice W [kg/kg]                          (686-822)
     rain(:),     &! 7) Rain [kg/(m2 *s)]                        (823-959)
     snow(:),     &! 8) Snow [kg/(m2 *s)]                        (960-1096)
     w(:),        &! 9) Vertical Velocity [Pa/s]                 (1097-1233)
     lnpsurf,     &!10) Ln of Surf Pressure in [Pa]              (1234)
     z0,          &!11) Surface geopotential [m2/s2]             (1235) 
     tsurf,       &!12) Surface Skin Temperature [K]             (1236)
     t2m,         &!13) 2m Temperature [K]                       (1237)
     td2m,        &!14) 2m Dew point temperature [K]             (1238)
     u10,         &!15) 10m wind speed U component [m/s]         (1239)
     v10,         &!16) 10m wind speed V component [m/s]         (1240)
     stratrsrf,   &!17) Stratiform precipitation at surface [m]  (1241)
     convrsrf,    &!18) Convective precipitation at surface [m]  (1242)
     snowsurf,    &!19) Snowfall at surface [m] (water equival.) (1243)
     lsm,         &!20) Land/sea Mask [0-1]                      (1244)
     lat,         &!21) Latitude [deg]                           (1245)
     long,        &!22) Longitude [deg]                          (1246)
     year,        &!23) Year                                     (1247)
     month,       &!24) Month                                    (1248)
     day,         &!25) Day                                      (1249)
     step,        &!26) Step                                     (1250)
     gpoint,      &!27) Grid point [1-2140702]                   (1251)
     ind           !28) Index (rank-sorted)                      (1252) 

  read(2,'(26(e13.6,x),i5,3i3,i8,i5)',end=1) &
     alb,         &! 1) Albedo [0-1]                             (1)
     sr,          &! 2) Roughness [m]                            (2)
     tvl,         &! 3) Low  vegetation type [0-20]              (3)
     tvh,         &! 4) High vegetation type [0-19]              (4)
     cvl,         &! 5) Low  vegetation fractional cover [0-1]   (5)
     cvh,         &! 6) High vegetation fractional cover [0-1]   (6)
     seaice,      &! 7) SeaIce cover [0-1]                       (7)
     asn,         &! 8) Snow albedo [0-1]                        (8)
     rsn,         &! 9) Snow density [kg/m3]                     (9)
     tsn,         &!10) Snow temperature [K]                    (10)
     sd,          &!11) Snow depth [m]                          (11)
     stl1,        &!12) Soil Layer 1 temperature [K]            (12)
     stl2,        &!13) Soil Layer 2 temperature [K]            (13)
     stl3,        &!14) Soil Layer 3 temperature [K]            (14)
     stl4,        &!15) Soil Layer 4 temperature [K]            (15)
     swvl1,       &!16) Volumetric Soil Water Layer 1 [m3/m3]   (16)
     swvl2,       &!17) Volumetric Soil Water Layer 2 [m3/m3]   (17)
     swvl3,       &!18) Volumetric Soil Water Layer 3 [m3/m3]   (18)
     swvl4,       &!19) Volumetric Soil Water Layer 4 [m3/m3]   (19)
     istl1,       &!20) Ice temperature Layer 1 [K]             (20)
     istl2,       &!21) Ice temperature Layer 2 [K]             (21)
     istl3,       &!22) Ice temperature Layer 3 [K]             (22)
     istl4,       &!23) Ice temperature Layer 4 [K]             (23)
! The following variables also appear in .atm file
     lsm,         &!%24) Land/sea Mask [0-1]                    (24)
     lat,         &!%25) Latitude [deg]                         (25)
     long,        &!%26) Longitude[deg]                         (26)
     year,        &!%27) Year of start of forecast              (27)
     month,       &!%28) Month of start of forecast             (28)
     day,         &!%29) Day of start of forecast               (29)
     step,        &!%30) Forecast step [hours]                  (30)
     gpoint,      &!%31) Grid point [1-2140702]                 (31)
     ind           !%32) Index (rank-sorted)                    (32) 

  ! compute surface pressure [Pa]
  psurf = exp(lnpsurf)

  ! compute surface geometric height [m]
  elev = z0 / 9.80665

  ! compute full-level pressure (pap) and half-level pressures (paph) [Pa]
  ! note that all profile variables above are given at full levels
  call ec_p137l(psurf,pap,paph)
  i = i + 1
enddo
1 write(*,*) 'Number of profiles found in the files:',i

end

! ----------------------------------------
!
      SUBROUTINE EC_P137l(spres,pap,paph)
!
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2014, EUMETSAT, All Rights Reserved.
!
!     Description:
!     Computes the 137-level vertical pressure grid
!       associated to the input surface pressure
!     All pressures are in Pa

      implicit none
      integer, parameter    :: nlev=137
      integer               :: jk
      real      :: spres
      real      :: aam(nlev+1), bbm(nlev+1)
      real      :: pap(nlev), paph(nlev+1)

      data aam / &
         0.000000, &
         2.000365, &
         3.102241, &
         4.666084, &
         6.827977, &
         9.746966, &
        13.605424, &
        18.608931, &
        24.985718, &
        32.985710, &
        42.879242, &
        54.955463, &
        69.520576, &
        86.895882, &
       107.415741, &
       131.425507, &
       159.279404, &
       191.338562, &
       227.968948, &
       269.539581, &
       316.420746, &
       368.982361, &
       427.592499, &
       492.616028, &
       564.413452, &
       643.339905, &
       729.744141, &
       823.967834, &
       926.344910, &
      1037.201172, &
      1156.853638, &
      1285.610352, &
      1423.770142, &
      1571.622925, &
      1729.448975, &
      1897.519287, &
      2076.095947, &
      2265.431641, &
      2465.770508, &
      2677.348145, &
      2900.391357, &
      3135.119385, &
      3381.743652, &
      3640.468262, &
      3911.490479, &
      4194.930664, &
      4490.817383, &
      4799.149414, &
      5119.895020, &
      5452.990723, &
      5798.344727, &
      6156.074219, &
      6526.946777, &
      6911.870605, &
      7311.869141, &
      7727.412109, &
      8159.354004, &
      8608.525391, &
      9076.400391, &
      9562.682617, &
     10065.978516, &
     10584.631836, &
     11116.662109, &
     11660.067383, &
     12211.547852, &
     12766.873047, &
     13324.668945, &
     13881.331055, &
     14432.139648, &
     14975.615234, &
     15508.256836, &
     16026.115234, &
     16527.322266, &
     17008.789062, &
     17467.613281, &
     17901.621094, &
     18308.433594, &
     18685.718750, &
     19031.289062, &
     19343.511719, &
     19620.042969, &
     19859.390625, &
     20059.931641, &
     20219.664062, &
     20337.863281, &
     20412.308594, &
     20442.078125, &
     20425.718750, &
     20361.816406, &
     20249.511719, &
     20087.085938, &
     19874.025391, &
     19608.572266, &
     19290.226562, &
     18917.460938, &
     18489.707031, &
     18006.925781, &
     17471.839844, &
     16888.687500, &
     16262.046875, &
     15596.695312, &
     14898.453125, &
     14173.324219, &
     13427.769531, &
     12668.257812, &
     11901.339844, &
     11133.304688, &
     10370.175781, &
      9617.515625, &
      8880.453125, &
      8163.375000, &
      7470.343750, &
      6804.421875, &
      6168.531250, &
      5564.382812, &
      4993.796875, &
      4457.375000, &
      3955.960938, &
      3489.234375, &
      3057.265625, &
      2659.140625, &
      2294.242188, &
      1961.500000, &
      1659.476562, &
      1387.546875, &
      1143.250000, &
       926.507812, &
       734.992188, &
       568.062500, &
       424.414062, &
       302.476562, &
       202.484375, &
       122.101562, &
        62.781250, &
        22.835938, &
         3.757813, &
         0.000000, &
         0.000000 /

      data bbm / &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000000, &
     0.000007, &
     0.000024, &
     0.000059, &
     0.000112, &
     0.000199, &
     0.000340, &
     0.000562, &
     0.000890, &
     0.001353, &
     0.001992, &
     0.002857, &
     0.003971, &
     0.005378, &
     0.007133, &
     0.009261, &
     0.011806, &
     0.014816, &
     0.018318, &
     0.022355, &
     0.026964, &
     0.032176, &
     0.038026, &
     0.044548, &
     0.051773, &
     0.059728, &
     0.068448, &
     0.077958, &
     0.088286, &
     0.099462, &
     0.111505, &
     0.124448, &
     0.138313, &
     0.153125, &
     0.168910, &
     0.185689, &
     0.203491, &
     0.222333, &
     0.242244, &
     0.263242, &
     0.285354, &
     0.308598, &
     0.332939, &
     0.358254, &
     0.384363, &
     0.411125, &
     0.438391, &
     0.466003, &
     0.493800, &
     0.521619, &
     0.549301, &
     0.576692, &
     0.603648, &
     0.630036, &
     0.655736, &
     0.680643, &
     0.704669, &
     0.727739, &
     0.749797, &
     0.770798, &
     0.790717, &
     0.809536, &
     0.827256, &
     0.843881, &
     0.859432, &
     0.873929, &
     0.887408, &
     0.899900, &
     0.911448, &
     0.922096, &
     0.931881, &
     0.940860, &
     0.949064, &
     0.956550, &
     0.963352, &
     0.969513, &
     0.975078, &
     0.980072, &
     0.984542, &
     0.988500, &
     0.991984, &
     0.995003, &
     0.997630, &
     1.000000 /

      do jk=1,nlev+1
      paph(jk)=aam(jk)+bbm(jk)*spres
      end do
      do jk=1,nlev
      pap(jk)=0.5*(paph(jk)+paph(jk+1))
      end do

      return
      end subroutine ec_p137l
